-- The SQL*Plus script fundamental_user_schema.sql performs the following:
--   1. Creates the fundamental_user.
--   2. Creates the database objects.
--   3. Populates the database tables with example data.

-- This script should be run by the system user (or the DBA).
CONNECT system/manager;

-- drop fundamental_user
DROP USER fundamental_user CASCADE;

-- create fundamental_user
CREATE USER fundamental_user IDENTIFIED BY fundamental_password;

-- allow fundamental_user to connect and create database objects
GRANT connect, resource TO fundamental_user;

-- connect as the fundamental_user
CONNECT fundamental_user/fundamental_password;

-- create the tables
CREATE TABLE customers (
  id         NUMBER CONSTRAINT customers_pk PRIMARY KEY,
  first_name VARCHAR2(50) NOT NULL,
  last_name  VARCHAR2(50) NOT NULL,
  dob        DATE,
  phone      VARCHAR2(20)
);

CREATE TABLE product_types (
  id   NUMBER CONSTRAINT product_types_pk PRIMARY KEY,
  name VARCHAR2(50) NOT NULL
);

CREATE TABLE products (
  id          NUMBER CONSTRAINT products_pk PRIMARY KEY,
  type_id     NUMBER
    CONSTRAINT products_fk_product_types
    REFERENCES product_types(id),
  name        VARCHAR2(50) NOT NULL,
  description VARCHAR2(500),
  price       NUMBER(10, 2)
);

CREATE TABLE purchases (
  product_id   NUMBER
    CONSTRAINT purchases_fk_products
    REFERENCES products(id),
  purchased_by NUMBER
    CONSTRAINT purchases_fk_customers
    REFERENCES customers(id),
  quantity     NUMBER NOT NULL,
  CONSTRAINT   purchases_pk PRIMARY KEY (product_id, purchased_by)
);

CREATE TABLE price_audit (
  product_id NUMBER CONSTRAINT price_audit_fk_products REFERENCES products(id),
  old_price  NUMBER(10, 2),
  new_price  NUMBER(10, 2)
);

-- create the functions, procedures and packages

-- procedure update_product_price() updates the price of
-- a product by a given factor (both specified as parameters)
CREATE PROCEDURE update_product_price(

  p_product_id IN products.id%TYPE,
  p_factor     IN NUMBER

) AS

  product_count INTEGER;

BEGIN

  -- ensure that the product exists
  SELECT
    COUNT(*)
  INTO
    product_count
  FROM
    products
  WHERE
    id = p_product_id;

  -- if the product doesn't exist then return
  IF product_count = 0 THEN
    RETURN;
  END IF;

  -- if the product does exist then perform the update
  UPDATE
    products
  SET
    price = price * p_factor;

  -- commit the transaction
  COMMIT;

END update_product_price;
/


-- function update_product_price_func() updates the price of
-- a product by a given factor (both specified as a parameters),
-- and returns 0 if successful, otherwise returns 1 (indicating that the
-- product was not found)
CREATE FUNCTION update_product_price_func(

  p_product_id IN products.id%TYPE,
  p_factor     IN NUMBER

) RETURN INTEGER AS

  product_count INTEGER;

BEGIN

  -- ensure that the product exists
  SELECT
    COUNT(*)
  INTO
    product_count
  FROM
    products
  WHERE
    id = p_product_id;

  -- if the product doesn't exist then return 1
  IF product_count = 0 THEN

    -- return 1, indicating that the function was unsuccessful
    RETURN 1;

  END IF;

  -- if the product does exist then perform the update
  UPDATE
    products
  SET
    price = price * p_factor;

  -- commit the transaction
  COMMIT;

  -- return 0, indication that the function was successful
  RETURN 0;

END update_product_price_func;
/


-- package ref_cursor_package illustrates the use of the
-- REF CURSOR type
CREATE PACKAGE ref_cursor_package AS

  TYPE t_ref_cursor IS REF CURSOR;
  FUNCTION get_customers_ref_cursor RETURN t_ref_cursor;

END ref_cursor_package;
/

CREATE PACKAGE BODY ref_cursor_package AS

  -- function get_customers_ref_cursor() returns a REF CURSOR
  FUNCTION get_customers_ref_cursor
  RETURN t_ref_cursor IS

    customers_ref_cursor t_ref_cursor;

  BEGIN

    -- get the REF CURSOR
    OPEN customers_ref_cursor FOR
      SELECT
        id, first_name, last_name, dob, phone
      FROM
        customers;

    -- return the REF CURSOR
    RETURN customers_ref_cursor;

  END get_customers_ref_cursor;

END ref_cursor_package;
/


-- insert sample data into customers table

INSERT INTO customers (id, first_name, last_name, dob, phone)
VALUES (1, 'John', 'Smith', '01-JAN-1965', '650-555-1212');

INSERT INTO customers (id, first_name, last_name, dob, phone)
VALUES (2, 'Cynthia', 'Stevens', '05-MAR-1968', '650-555-1213');

INSERT INTO customers (id, first_name, last_name, dob, phone)
VALUES (3, 'Steve', 'Seymour', '16-JUN-1971', '650-555-1214');

INSERT INTO customers (id, first_name, last_name, dob, phone)
VALUES (4, 'Gail', 'Williams', '01-DEC-1975', '650-555-1215');

INSERT INTO customers (id, first_name, last_name, dob, phone)
VALUES (5, 'Doreen', 'Heyson', '20-AUG-1970', '650-555-1216');


-- insert sample data into product_types table

INSERT INTO product_types (id, name)
VALUES (1, 'Book');

INSERT INTO product_types (id, name)
VALUES (2, 'Video');

INSERT INTO product_types (id, name)
VALUES (3, 'DVD');

INSERT INTO product_types (id, name)
VALUES (4, 'CD');


-- insert sample data into products table

INSERT INTO products (id, type_id, name, description, price)
VALUES (1, 1, 'Beyond Understanding', 'The frontiers of human knowledge', 19.95);

INSERT INTO products (id, type_id, name, description, price)
VALUES (2, 1, 'Physics', 'Fundamental understanding of the natural world', 30.00);

INSERT INTO products (id, type_id, name, description, price)
VALUES (3, 1, 'Star Travelers', 'The adventures of futuristic astronauts', 25.99);

INSERT INTO products (id, type_id, name, description, price)
VALUES (4, 2, 'Seventh Sense', 'Supernatural tale of the unknown', 13.95);

INSERT INTO products (id, type_id, name, description, price)
VALUES (5, 2, 'Quantum Jump', 'The physics of the quantum world', 49.99);

INSERT INTO products (id, type_id, name, description, price)
VALUES (6, 2, '2002: A Space Journey', 'The year after 2001', 14.95);

INSERT INTO products (id, type_id, name, description, price)
VALUES (7, 3, 'Super Force', 'The adventures of Super Force Man', 13.49);

INSERT INTO products (id, type_id, name, description, price)
VALUES (8, 3, 'The Man from Another Planet', 'A man from another planet lands on Earth', 12.99);

INSERT INTO products (id, type_id, name, description, price)
VALUES (9, 4, 'Classical Music', 'The best classical music', 10.99);

INSERT INTO products (id, type_id, name, description, price)
VALUES (10, 4, 'Pop 2000', 'The best popular music from 2000', 15.99);

INSERT INTO products (id, type_id, name, description, price)
VALUES (11, 4, 'Creative Riot', 'Debut album', 14.99);

INSERT INTO products (id, type_id, name, description, price)
VALUES (12, 4, 'Front Line', 'Their greatest hits', 13.49);

-- insert sample data into purchases table

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 1, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 2, 3);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 3, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 4, 2);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 5, 4);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 6, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 7, 3);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 8, 2);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 9, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 10, 2);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 11, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (1, 12, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (2, 1, 2);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (2, 2, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (2, 3, 3);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (2, 4, 2);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (2, 5, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (3, 1, 4);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (3, 2, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (3, 3, 2);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (3, 4, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (3, 5, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (4, 1, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (4, 2, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (4, 3, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (5, 1, 1);

INSERT INTO purchases (purchased_by, product_id, quantity)
VALUES (5, 2, 1);

-- commit the transaction
COMMIT;